from __future__ import annotations

import datetime as dt
import sqlite3
from collections.abc import Generator
from math import e, pi, sqrt
from typing import Any

import pytest
from pytest_lazy_fixtures import lf

import prettytable
from prettytable import (
    HRuleStyle,
    PrettyTable,
    RowType,
    TableStyle,
    VRuleStyle,
    from_db_cursor,
)


def test_version() -> None:
    assert isinstance(prettytable.__version__, str)
    assert prettytable.__version__[0].isdigit()
    assert prettytable.__version__.count(".") >= 2
    assert prettytable.__version__[-1].isdigit()


# Australian capital city data example table
CITY_DATA_HEADER = ["City name", "Area", "Population", "Annual Rainfall"]
CITY_DATA = [
    ["Adelaide", 1295, 1158259, 600.5],
    ["Brisbane", 5905, 1857594, 1146.4],
    ["Darwin", 112, 120900, 1714.7],
    ["Hobart", 1357, 205556, 619.5],
    ["Sydney", 2058, 4336374, 1214.8],
    ["Melbourne", 1566, 3806092, 646.9],
    ["Perth", 5386, 1554769, 869.4],
]


class TestNoneOption:
    def test_none_char_valid_option(self) -> None:
        PrettyTable(["Field 1", "Field 2", "Field 3"], none_format="")

    def test_none_char_invalid_option(self) -> None:
        with pytest.raises(TypeError) as exc:
            PrettyTable(["Field 1", "Field 2", "Field 3"], none_format=2)
        assert "must be a string" in str(exc.value)

    def test_no_value_replace_none(self) -> None:
        table = PrettyTable(["Field 1", "Field 2", "Field 3"])
        table.add_row(["value 1", None, "value 2"])
        assert (
            table.get_string().strip()
            == """
+---------+---------+---------+
| Field 1 | Field 2 | Field 3 |
+---------+---------+---------+
| value 1 |   None  | value 2 |
+---------+---------+---------+
""".strip()
        )

    def test_no_value_replace_none_with_default_field_names(self) -> None:
        table = PrettyTable()
        table.add_row(["value 1", "None", "value 2"])
        assert (
            table.get_string().strip()
            == """
+---------+---------+---------+
| Field 1 | Field 2 | Field 3 |
+---------+---------+---------+
| value 1 |   None  | value 2 |
+---------+---------+---------+
""".strip()
        )

    def test_replace_none_all(self) -> None:
        table = PrettyTable(
            ["Field 1", "Field 2", "Field 3", "Field 4"], none_format="N/A"
        )
        table.add_row(["value 1", None, "None", ""])
        assert (
            table.get_string().strip()
            == """
+---------+---------+---------+---------+
| Field 1 | Field 2 | Field 3 | Field 4 |
+---------+---------+---------+---------+
| value 1 |   N/A   |   N/A   |         |
+---------+---------+---------+---------+
""".strip()
        )

    def test_replace_none_by_col(self) -> None:
        table = PrettyTable(["Field 1", "Field 2", "Field 3"])
        table.none_format["Field 2"] = "N/A"
        table.none_format["Field 3"] = ""
        table.add_row(["value 1", None, None])
        assert (
            table.get_string().strip()
            == """
+---------+---------+---------+
| Field 1 | Field 2 | Field 3 |
+---------+---------+---------+
| value 1 |   N/A   |         |
+---------+---------+---------+
""".strip()
        )

    def test_replace_none_recompute_width(self) -> None:
        table = PrettyTable()
        table.add_row([None])
        table.none_format = "0123456789"
        assert (
            table.get_string().strip()
            == """
+------------+
|  Field 1   |
+------------+
| 0123456789 |
+------------+
""".strip()
        )

    def test_replace_none_maintain_width_on_recompute(self) -> None:
        table = PrettyTable()
        table.add_row(["Hello"])
        table.none_format = "0123456789"
        assert (
            table.get_string().strip()
            == """
+---------+
| Field 1 |
+---------+
|  Hello  |
+---------+
""".strip()
        )

    def test_replace_none_recompute_width_multi_column(self) -> None:
        table = PrettyTable()
        table.add_row(["Hello", None, "World"])
        table.none_format = "0123456789"
        assert (
            table.get_string().strip()
            == """
+---------+------------+---------+
| Field 1 |  Field 2   | Field 3 |
+---------+------------+---------+
|  Hello  | 0123456789 |  World  |
+---------+------------+---------+
""".strip()
        )


class TestBuildEquivalence:
    """Make sure that building a table row-by-row and column-by-column yield the same
    results"""

    @pytest.mark.parametrize(
        ["left_hand", "right_hand"],
        [
            (
                lf("row_prettytable"),
                lf("col_prettytable"),
            ),
            (
                lf("row_prettytable"),
                lf("mix_prettytable"),
            ),
        ],
    )
    def test_equivalence_ascii(
        self, left_hand: PrettyTable, right_hand: PrettyTable
    ) -> None:
        assert left_hand.get_string() == right_hand.get_string()

    @pytest.mark.parametrize(
        ["left_hand", "right_hand"],
        [
            (
                lf("row_prettytable"),
                lf("col_prettytable"),
            ),
            (
                lf("row_prettytable"),
                lf("mix_prettytable"),
            ),
        ],
    )
    def test_equivalence_html(
        self, left_hand: PrettyTable, right_hand: PrettyTable
    ) -> None:
        assert left_hand.get_html_string() == right_hand.get_html_string()

    @pytest.mark.parametrize(
        ["left_hand", "right_hand"],
        [
            (
                lf("row_prettytable"),
                lf("col_prettytable"),
            ),
            (
                lf("row_prettytable"),
                lf("mix_prettytable"),
            ),
        ],
    )
    def test_equivalence_latex(
        self, left_hand: PrettyTable, right_hand: PrettyTable
    ) -> None:
        assert left_hand.get_latex_string() == right_hand.get_latex_string()

    @pytest.mark.parametrize(
        ["left_hand", "right_hand"],
        [
            (
                lf("row_prettytable"),
                lf("col_prettytable"),
            ),
            (
                lf("row_prettytable"),
                lf("mix_prettytable"),
            ),
        ],
    )
    def test_equivalence_mediawiki(
        self, left_hand: PrettyTable, right_hand: PrettyTable
    ) -> None:
        assert left_hand.get_mediawiki_string() == right_hand.get_mediawiki_string()


class TestDelete:
    def test_delete_column(self, col_prettytable: PrettyTable) -> None:
        col_prettytable.del_column("Area")

        assert (
            col_prettytable.get_string()
            == """+-----------+------------+-----------------+
| City name | Population | Annual Rainfall |
+-----------+------------+-----------------+
|  Adelaide |  1158259   |      600.5      |
|  Brisbane |  1857594   |      1146.4     |
|   Darwin  |   120900   |      1714.7     |
|   Hobart  |   205556   |      619.5      |
|   Sydney  |  4336374   |      1214.8     |
| Melbourne |  3806092   |      646.9      |
|   Perth   |  1554769   |      869.4      |
+-----------+------------+-----------------+"""
        )

    def test_delete_illegal_column_raises_error(
        self, col_prettytable: PrettyTable
    ) -> None:
        with pytest.raises(ValueError):
            col_prettytable.del_column("City not-a-name")

    def test_delete_row(self, city_data: PrettyTable) -> None:
        city_data.del_row(2)

        assert (
            city_data.get_string()
            == """+-----------+------+------------+-----------------+
| City name | Area | Population | Annual Rainfall |
+-----------+------+------------+-----------------+
|  Adelaide | 1295 |  1158259   |      600.5      |
|  Brisbane | 5905 |  1857594   |      1146.4     |
|   Hobart  | 1357 |   205556   |      619.5      |
|   Sydney  | 2058 |  4336374   |      1214.8     |
| Melbourne | 1566 |  3806092   |      646.9      |
|   Perth   | 5386 |  1554769   |      869.4      |
+-----------+------+------------+-----------------+"""
        )

    def test_delete_row_unavailable(self, city_data: PrettyTable) -> None:
        with pytest.raises(IndexError):
            city_data.del_row(10)


class TestFieldNameLessTable:
    """Make sure that building and stringing a table with no fieldnames works fine"""

    def test_can_string_ascii(self, field_name_less_table: PrettyTable) -> None:
        output = field_name_less_table.get_string()
        assert "|  Field 1  | Field 2 | Field 3 | Field 4 |" in output
        assert "|  Adelaide |   1295  | 1158259 |  600.5  |" in output

    def test_can_string_html(self, field_name_less_table: PrettyTable) -> None:
        output = field_name_less_table.get_html_string()
        assert "<th>Field 1</th>" in output
        assert "<td>Adelaide</td>" in output

    def test_can_string_latex(self, field_name_less_table: PrettyTable) -> None:
        output = field_name_less_table.get_latex_string()
        assert "Field 1 & Field 2 & Field 3 & Field 4 \\\\" in output
        assert "Adelaide & 1295 & 1158259 & 600.5 \\\\" in output

    def test_can_string_mediawiki(self, field_name_less_table: PrettyTable) -> None:
        output = field_name_less_table.get_mediawiki_string(header=True)
        assert "! Field 1 !! Field 2 !! Field 3 !! Field 4" in output
        assert "| Adelaide || 1295 || 1158259 || 600.5" in output

    def test_add_field_names_later(self, field_name_less_table: PrettyTable) -> None:
        field_name_less_table.field_names = CITY_DATA_HEADER
        assert (
            "City name | Area | Population | Annual Rainfall"
            in field_name_less_table.get_string()
        )


@pytest.fixture(scope="function")
def aligned_before_table() -> PrettyTable:
    table = PrettyTable()
    table.align = "r"
    table.field_names = CITY_DATA_HEADER
    for row in CITY_DATA:
        table.add_row(row)
    return table


@pytest.fixture(scope="function")
def aligned_after_table() -> PrettyTable:
    table = PrettyTable()
    table.field_names = CITY_DATA_HEADER
    for row in CITY_DATA:
        table.add_row(row)
    table.align = "r"
    return table


class TestAlignment:
    """Make sure alignment works regardless of when it was set"""

    def test_aligned_ascii(
        self, aligned_before_table: PrettyTable, aligned_after_table: PrettyTable
    ) -> None:
        assert aligned_before_table.get_string() == aligned_after_table.get_string()

    def test_aligned_html(
        self, aligned_before_table: PrettyTable, aligned_after_table: PrettyTable
    ) -> None:
        assert (
            aligned_before_table.get_html_string()
            == aligned_after_table.get_html_string()
        )

    def test_aligned_latex(
        self, aligned_before_table: PrettyTable, aligned_after_table: PrettyTable
    ) -> None:
        assert (
            aligned_before_table.get_latex_string()
            == aligned_after_table.get_latex_string()
        )

    def test_aligned_mediawiki(
        self, aligned_before_table: PrettyTable, aligned_after_table: PrettyTable
    ) -> None:
        assert aligned_before_table.get_mediawiki_string(
            header=True
        ) == aligned_after_table.get_mediawiki_string(header=True)


class TestOptionOverride:
    """Make sure all options are properly overwritten by get_string."""

    def test_border(self, city_data: PrettyTable) -> None:
        assert city_data.get_string() != city_data.get_string(border=False)

    def test_header(self, city_data: PrettyTable) -> None:
        assert city_data.get_string() != city_data.get_string(header=False)

    def test_hrules_all(self, city_data: PrettyTable) -> None:
        assert city_data.get_string() != city_data.get_string(hrules=HRuleStyle.ALL)

    def test_hrules_none(self, city_data: PrettyTable) -> None:
        assert city_data.get_string() != city_data.get_string(hrules=HRuleStyle.NONE)


class TestOptionAttribute:
    """Make sure all options which have an attribute interface work as they should.
    Also make sure option settings are copied correctly when a table is cloned by
    slicing."""

    def test_set_for_all_columns(self, city_data: PrettyTable) -> None:
        city_data.field_names = sorted(city_data.field_names)
        city_data.align = "l"
        city_data.max_width = 10
        city_data.start = 2
        city_data.end = 4
        city_data.sortby = "Area"
        city_data.reversesort = True
        city_data.header = True
        city_data.border = False
        city_data.hrules = HRuleStyle.ALL
        city_data.int_format = "4"
        city_data.float_format = "2.2"
        city_data.padding_width = 2
        city_data.left_padding_width = 2
        city_data.right_padding_width = 2
        city_data.vertical_char = "!"
        city_data.horizontal_char = "~"
        city_data.junction_char = "*"
        city_data.top_junction_char = "@"
        city_data.bottom_junction_char = "#"
        city_data.right_junction_char = "$"
        city_data.left_junction_char = "%"
        city_data.top_right_junction_char = "^"
        city_data.top_left_junction_char = "&"
        city_data.bottom_right_junction_char = "("
        city_data.bottom_left_junction_char = ")"
        city_data.format = True
        city_data.attributes = {"class": "prettytable"}
        assert city_data.get_string() == city_data[:].get_string()

    def test_set_for_one_column(self, city_data: PrettyTable) -> None:
        city_data.align["Rainfall"] = "l"
        city_data.max_width["Name"] = 10
        city_data.int_format["Population"] = "4"
        city_data.float_format["Area"] = "2.2"
        assert city_data.get_string() == city_data[:].get_string()

    def test_preserve_internal_border(self) -> None:
        table = PrettyTable(preserve_internal_border=True)
        assert table.preserve_internal_border is True

    def test_internal_border_preserved(self, helper_table: PrettyTable) -> None:
        helper_table.border = False
        helper_table.preserve_internal_border = True

        assert (
            helper_table.get_string().strip()
            == """
   | Field 1 | Field 2 | Field 3  
---+---------+---------+---------
 1 | value 1 |  value2 |  value3  
 4 | value 4 |  value5 |  value6  
 7 | value 7 |  value8 |  value9  
""".strip()  # noqa: W291
        )


@pytest.fixture(scope="module")
def db_cursor() -> Generator[sqlite3.Cursor]:
    conn = sqlite3.connect(":memory:")
    cur = conn.cursor()
    yield cur
    cur.close()
    conn.close()


@pytest.fixture(scope="module")
def init_db(db_cursor: sqlite3.Cursor) -> Generator[Any]:
    db_cursor.execute(
        "CREATE TABLE cities "
        "(name TEXT, area INTEGER, population INTEGER, rainfall REAL)"
    )
    for row in CITY_DATA:
        db_cursor.execute(f"INSERT INTO cities VALUES {tuple(row)}")
    yield
    db_cursor.execute("DROP TABLE cities")


class TestBasic:
    """Some very basic tests."""

    def test_table_rows(self, city_data: PrettyTable) -> None:
        rows = city_data.rows
        assert len(rows) == 7
        assert rows[0] == CITY_DATA[0]

    def test_add_rows(self, city_data: PrettyTable) -> None:
        """A table created with multiple add_row calls is the same as one created
        with a single add_rows
        """
        table = PrettyTable(CITY_DATA_HEADER)
        table.add_rows(CITY_DATA)
        assert str(city_data) == str(table)

        table.add_rows([])
        assert str(city_data) == str(table)

    def _test_no_blank_lines(self, table: PrettyTable) -> None:
        string = table.get_string()
        lines = string.split("\n")
        assert "" not in lines

    def _test_all_length_equal(self, table: PrettyTable) -> None:
        string = table.get_string()
        lines = string.split("\n")
        lengths = {len(line) for line in lines}
        assert len(lengths) == 1

    def test_no_blank_lines(self, city_data: PrettyTable) -> None:
        """No table should ever have blank lines in it."""
        self._test_no_blank_lines(city_data)

    def test_all_lengths_equal(self, city_data: PrettyTable) -> None:
        """All lines in a table should be of the same length."""
        self._test_all_length_equal(city_data)

    def test_no_blank_lines_with_title(self, city_data: PrettyTable) -> None:
        """No table should ever have blank lines in it."""
        city_data.title = "My table"
        self._test_no_blank_lines(city_data)

    def test_all_lengths_equal_with_title(self, city_data: PrettyTable) -> None:
        """All lines in a table should be of the same length."""
        city_data.title = "My table"
        self._test_all_length_equal(city_data)

    def test_all_lengths_equal_with_long_title(self, city_data: PrettyTable) -> None:
        """All lines in a table should be of the same length, even with a long title."""
        city_data.title = "My table (75 characters wide) " + "=" * 45
        self._test_all_length_equal(city_data)

    def test_no_blank_lines_without_border(self, city_data: PrettyTable) -> None:
        """No table should ever have blank lines in it."""
        city_data.border = False
        self._test_no_blank_lines(city_data)

    def test_all_lengths_equal_without_border(self, city_data: PrettyTable) -> None:
        """All lines in a table should be of the same length."""
        city_data.border = False
        self._test_all_length_equal(city_data)

    def test_no_blank_lines_without_header(self, city_data: PrettyTable) -> None:
        """No table should ever have blank lines in it."""
        city_data.header = False
        self._test_no_blank_lines(city_data)

    def test_all_lengths_equal_without_header(self, city_data: PrettyTable) -> None:
        """All lines in a table should be of the same length."""
        city_data.header = False
        self._test_all_length_equal(city_data)

    def test_no_blank_lines_with_hrules_none(self, city_data: PrettyTable) -> None:
        """No table should ever have blank lines in it."""
        city_data.hrules = HRuleStyle.NONE
        self._test_no_blank_lines(city_data)

    def test_all_lengths_equal_with_hrules_none(self, city_data: PrettyTable) -> None:
        """All lines in a table should be of the same length."""
        city_data.hrules = HRuleStyle.NONE
        self._test_all_length_equal(city_data)

    def test_no_blank_lines_with_hrules_all(self, city_data: PrettyTable) -> None:
        """No table should ever have blank lines in it."""
        city_data.hrules = HRuleStyle.ALL
        self._test_no_blank_lines(city_data)

    def test_all_lengths_equal_with_hrules_all(self, city_data: PrettyTable) -> None:
        """All lines in a table should be of the same length."""
        city_data.hrules = HRuleStyle.ALL
        self._test_all_length_equal(city_data)

    def test_no_blank_lines_with_style_msword(self, city_data: PrettyTable) -> None:
        """No table should ever have blank lines in it."""
        city_data.set_style(TableStyle.MSWORD_FRIENDLY)
        self._test_no_blank_lines(city_data)

    def test_all_lengths_equal_with_style_msword(self, city_data: PrettyTable) -> None:
        """All lines in a table should be of the same length."""
        city_data.set_style(TableStyle.MSWORD_FRIENDLY)
        self._test_all_length_equal(city_data)

    def test_no_blank_lines_with_int_format(self, city_data: PrettyTable) -> None:
        """No table should ever have blank lines in it."""
        city_data.int_format = "04"
        self._test_no_blank_lines(city_data)

    def test_all_lengths_equal_with_int_format(self, city_data: PrettyTable) -> None:
        """All lines in a table should be of the same length."""
        city_data.int_format = "04"
        self._test_all_length_equal(city_data)

    def test_no_blank_lines_with_float_format(self, city_data: PrettyTable) -> None:
        """No table should ever have blank lines in it."""
        city_data.float_format = "6.2f"
        self._test_no_blank_lines(city_data)

    def test_all_lengths_equal_with_float_format(self, city_data: PrettyTable) -> None:
        """All lines in a table should be of the same length."""
        city_data.float_format = "6.2f"
        self._test_all_length_equal(city_data)

    def test_no_blank_lines_from_csv(self, city_data_from_csv: PrettyTable) -> None:
        """No table should ever have blank lines in it."""
        self._test_no_blank_lines(city_data_from_csv)

    def test_all_lengths_equal_from_csv(self, city_data_from_csv: PrettyTable) -> None:
        """All lines in a table should be of the same length."""
        self._test_all_length_equal(city_data_from_csv)

    def test_no_blank_lines_from_mediawiki(
        self, city_data_from_mediawiki: PrettyTable
    ) -> None:
        """No table should ever have blank lines in it."""
        self._test_no_blank_lines(city_data_from_mediawiki)

    def test_all_lengths_equal_from_mediawiki(
        self, city_data_from_mediawiki: PrettyTable
    ) -> None:
        """All lines in a table should be of the same length."""
        self._test_all_length_equal(city_data_from_mediawiki)

    def test_rowcount(self, city_data: PrettyTable) -> None:
        assert city_data.rowcount == 7

    def test_colcount(self, city_data: PrettyTable) -> None:
        assert city_data.colcount == 4

    def test_getitem(self, city_data: PrettyTable) -> None:
        assert (
            city_data[1].get_string()
            == """+-----------+------+------------+-----------------+
| City name | Area | Population | Annual Rainfall |
+-----------+------+------------+-----------------+
|  Brisbane | 5905 |  1857594   |      1146.4     |
+-----------+------+------------+-----------------+"""
        )

    def test_invalid_getitem(self, city_data: PrettyTable) -> None:
        with pytest.raises(IndexError):
            assert city_data[10]

    @pytest.mark.usefixtures("init_db")
    def test_no_blank_lines_from_db(self, db_cursor: sqlite3.Cursor) -> None:
        """No table should ever have blank lines in it."""
        db_cursor.execute("SELECT * FROM cities")
        table = from_db_cursor(db_cursor)
        assert table is not None
        self._test_no_blank_lines(table)

    @pytest.mark.usefixtures("init_db")
    def test_all_lengths_equal_from_db(self, db_cursor: sqlite3.Cursor) -> None:
        """No table should ever have blank lines in it."""
        db_cursor.execute("SELECT * FROM cities")
        table = from_db_cursor(db_cursor)
        assert table is not None
        self._test_all_length_equal(table)


class TestEmptyTable:
    """Make sure the print_empty option works"""

    def test_print_empty_true(self, city_data: PrettyTable) -> None:
        table = PrettyTable()
        table.field_names = CITY_DATA_HEADER

        assert table.get_string(print_empty=True) != ""
        assert table.get_string(print_empty=True) != city_data.get_string(
            print_empty=True
        )

    def test_print_empty_false(self, city_data: PrettyTable) -> None:
        table = PrettyTable()
        table.field_names = CITY_DATA_HEADER

        assert table.get_string(print_empty=False) == ""
        assert table.get_string(print_empty=False) != city_data.get_string(
            print_empty=False
        )

    def test_interaction_with_border(self) -> None:
        table = PrettyTable()
        table.field_names = CITY_DATA_HEADER

        assert table.get_string(border=False, print_empty=True) == ""


class TestSlicing:
    def test_slice_all(self, city_data: PrettyTable) -> None:
        table = city_data[:]
        assert city_data.get_string() == table.get_string()

    def test_slice_first_two_rows(self, city_data: PrettyTable) -> None:
        table = city_data[0:2]
        string = table.get_string()
        assert len(string.split("\n")) == 6
        for row_index in (0, 1):
            city = CITY_DATA[row_index][0]
            assert isinstance(city, str)
            assert city in string
        for row_index in (2, 3, 4, 5, 6):
            city = CITY_DATA[row_index][0]
            assert isinstance(city, str)
            assert city not in string

    def test_slice_last_two_rows(self, city_data: PrettyTable) -> None:
        table = city_data[-2:]
        string = table.get_string()
        assert len(string.split("\n")) == 6
        for row_index in (0, 1, 2, 3, 4):
            city = CITY_DATA[row_index][0]
            assert isinstance(city, str)
            assert city not in string
        for row_index in (5, 6):
            city = CITY_DATA[row_index][0]
            assert isinstance(city, str)
            assert city in string


class TestRowFilter:
    EXPECTED_RESULT = """+-----------+------+------------+-----------------+
| City name | Area | Population | Annual Rainfall |
+-----------+------+------------+-----------------+
|  Adelaide | 1295 |  1158259   |      600.5      |
|  Brisbane | 5905 |  1857594   |      1146.4     |
|   Sydney  | 2058 |  4336374   |      1214.8     |
| Melbourne | 1566 |  3806092   |      646.9      |
|   Perth   | 5386 |  1554769   |      869.4      |
+-----------+------+------------+-----------------+"""

    def filter_function(self, vals: RowType) -> bool:
        return vals[2] > 999999

    def test_row_filter(self, city_data: PrettyTable) -> None:
        city_data.row_filter = self.filter_function
        assert city_data.row_filter == self.filter_function
        assert self.EXPECTED_RESULT == city_data.get_string()

    def test_row_filter_at_class_declaration(self) -> None:
        table = PrettyTable(
            field_names=CITY_DATA_HEADER,
            row_filter=self.filter_function,
        )
        for row in CITY_DATA:
            table.add_row(row)
        assert table.row_filter == self.filter_function
        assert self.EXPECTED_RESULT == table.get_string().strip()


@pytest.fixture(scope="function")
def float_pt() -> PrettyTable:
    table = PrettyTable(["Constant", "Value"])
    table.add_row(["Pi", pi])
    table.add_row(["e", e])
    table.add_row(["sqrt(2)", sqrt(2)])
    return table


class TestFloatFormat:
    def test_no_decimals(self, float_pt: PrettyTable) -> None:
        float_pt.float_format = ".0f"
        assert "." not in float_pt.get_string()

    def test_round_to_5dp(self, float_pt: PrettyTable) -> None:
        float_pt.float_format = ".5f"
        string = float_pt.get_string()
        assert "3.14159" in string
        assert "3.141592" not in string
        assert "2.71828" in string
        assert "2.718281" not in string
        assert "2.718282" not in string
        assert "1.41421" in string
        assert "1.414213" not in string

    def test_pad_with_2zeroes(self, float_pt: PrettyTable) -> None:
        float_pt.float_format = "06.2f"
        string = float_pt.get_string()
        assert "003.14" in string
        assert "002.72" in string
        assert "001.41" in string


class TestBreakLine:
    @pytest.mark.parametrize(
        ["rows", "hrule", "expected_result"],
        [
            (
                [["value 1", "value2\nsecond line"], ["value 3", "value4"]],
                HRuleStyle.ALL,
                """
+---------+-------------+
| Field 1 |   Field 2   |
+---------+-------------+
| value 1 |    value2   |
|         | second line |
+---------+-------------+
| value 3 |    value4   |
+---------+-------------+
""",
            ),
            (
                [
                    ["value 1", "value2\nsecond line"],
                    ["value 3\n\nother line", "value4\n\n\nvalue5"],
                ],
                HRuleStyle.ALL,
                """
+------------+-------------+
|  Field 1   |   Field 2   |
+------------+-------------+
|  value 1   |    value2   |
|            | second line |
+------------+-------------+
|  value 3   |    value4   |
|            |             |
| other line |             |
|            |    value5   |
+------------+-------------+
""",
            ),
            (
                [
                    ["value 1", "value2\nsecond line"],
                    ["value 3\n\nother line", "value4\n\n\nvalue5"],
                ],
                HRuleStyle.FRAME,
                """
+------------+-------------+
|  Field 1   |   Field 2   |
+------------+-------------+
|  value 1   |    value2   |
|            | second line |
|  value 3   |    value4   |
|            |             |
| other line |             |
|            |    value5   |
+------------+-------------+
""",
            ),
        ],
    )
    def test_break_line_ascii(
        self, rows: list[list[Any]], hrule: int, expected_result: str
    ) -> None:
        table = PrettyTable(["Field 1", "Field 2"])
        for row in rows:
            table.add_row(row)
        result = table.get_string(hrules=hrule)
        assert result.strip() == expected_result.strip()


class TestFromDB:
    @pytest.mark.usefixtures("init_db")
    def test_non_select_cursor(self, db_cursor: sqlite3.Cursor) -> None:
        db_cursor.execute(f"INSERT INTO cities VALUES {tuple(CITY_DATA[0])}")
        assert from_db_cursor(db_cursor) is None


class TestCsvOutput:
    def test_csv_output(self, helper_table: PrettyTable) -> None:
        assert helper_table.get_csv_string(delimiter="\t", header=False) == (
            "1\tvalue 1\tvalue2\tvalue3\r\n"
            "4\tvalue 4\tvalue5\tvalue6\r\n"
            "7\tvalue 7\tvalue8\tvalue9\r\n"
        )
        assert helper_table.get_csv_string() == (
            ",Field 1,Field 2,Field 3\r\n"
            "1,value 1,value2,value3\r\n"
            "4,value 4,value5,value6\r\n"
            "7,value 7,value8,value9\r\n"
        )
        options = {"fields": ["Field 1", "Field 3"]}
        assert helper_table.get_csv_string(**options) == (
            "Field 1,Field 3\r\n"
            "value 1,value3\r\n"
            "value 4,value6\r\n"
            "value 7,value9\r\n"
        )


def test_paginate(city_data: PrettyTable) -> None:
    expected_page_1 = """
+-----------+------+------------+-----------------+
| City name | Area | Population | Annual Rainfall |
+-----------+------+------------+-----------------+
|  Adelaide | 1295 |  1158259   |      600.5      |
|  Brisbane | 5905 |  1857594   |      1146.4     |
|   Darwin  | 112  |   120900   |      1714.7     |
|   Hobart  | 1357 |   205556   |      619.5      |
+-----------+------+------------+-----------------+""".strip()
    expected_page_2 = """
+-----------+------+------------+-----------------+
| City name | Area | Population | Annual Rainfall |
+-----------+------+------------+-----------------+
|   Sydney  | 2058 |  4336374   |      1214.8     |
| Melbourne | 1566 |  3806092   |      646.9      |
|   Perth   | 5386 |  1554769   |      869.4      |
+-----------+------+------------+-----------------+""".strip()

    paginated = city_data.paginate(page_length=4).strip()
    assert paginated.startswith(expected_page_1)
    assert "\f" in paginated
    assert paginated.endswith(expected_page_2)

    paginated = city_data.paginate(page_length=4, line_break="\n")
    assert "\f" not in paginated
    assert "\n" in paginated


def test_autoindex(city_data: PrettyTable) -> None:
    """Testing that a table with a custom index row is
    equal to the one produced by the function
    .add_autoindex()
    """
    city_data.field_names = CITY_DATA_HEADER
    city_data.add_autoindex(fieldname="Test")

    table2 = PrettyTable()
    table2.field_names = ["Test"] + CITY_DATA_HEADER
    for idx, row in enumerate(CITY_DATA):
        table2.add_row([idx + 1] + row)

    assert str(city_data) == str(table2)


@pytest.fixture(scope="function")
def unpadded_pt() -> PrettyTable:
    table = PrettyTable(header=False, padding_width=0)
    table.add_row(list("abc"))
    table.add_row(list("def"))
    table.add_row(list("g.."))
    return table


class TestUnpaddedTable:
    def test_unbordered(self, unpadded_pt: PrettyTable) -> None:
        unpadded_pt.border = False
        result = unpadded_pt.get_string()
        expected = """
abc
def
g..
"""
        assert result.strip() == expected.strip()

    def test_bordered(self, unpadded_pt: PrettyTable) -> None:
        unpadded_pt.border = True
        result = unpadded_pt.get_string()
        expected = """
+-+-+-+
|a|b|c|
|d|e|f|
|g|.|.|
+-+-+-+
"""
        assert result.strip() == expected.strip()


class TestCustomFormatter:
    def test_init_custom_format_is_empty(self) -> None:
        table = PrettyTable()
        assert table.custom_format == {}

    def test_init_custom_format_set_value(self) -> None:
        table = PrettyTable(
            custom_format={"col1": (lambda col_name, value: f"{value:.2}")}
        )
        assert len(table.custom_format) == 1

    def test_init_custom_format_throw_error_is_not_callable(self) -> None:
        with pytest.raises(ValueError) as e:
            PrettyTable(custom_format={"col1": "{:.2}"})

        assert "Invalid value for custom_format.col1. Must be a function." in str(
            e.value
        )

    def test_can_set_custom_format_from_property_setter(self) -> None:
        table = PrettyTable()
        table.custom_format = {"col1": (lambda col_name, value: f"{value:.2}")}
        assert len(table.custom_format) == 1

    def test_set_custom_format_to_none_set_empty_dict(self) -> None:
        table = PrettyTable()
        table.custom_format = None
        assert isinstance(table.custom_format, dict)

    def test_set_custom_format_invalid_type_throw_error(self) -> None:
        table = PrettyTable()
        with pytest.raises(TypeError) as e:
            table.custom_format = "Some String"
        assert "The custom_format property need to be a dictionary or callable" in str(
            e.value
        )

    def test_use_custom_formatter_for_int(self, city_data: PrettyTable) -> None:
        city_data.custom_format["Annual Rainfall"] = lambda n, v: f"{v:.2f}"
        assert (
            city_data.get_string().strip()
            == """
+-----------+------+------------+-----------------+
| City name | Area | Population | Annual Rainfall |
+-----------+------+------------+-----------------+
|  Adelaide | 1295 |  1158259   |      600.50     |
|  Brisbane | 5905 |  1857594   |     1146.40     |
|   Darwin  | 112  |   120900   |     1714.70     |
|   Hobart  | 1357 |   205556   |      619.50     |
|   Sydney  | 2058 |  4336374   |     1214.80     |
| Melbourne | 1566 |  3806092   |      646.90     |
|   Perth   | 5386 |  1554769   |      869.40     |
+-----------+------+------------+-----------------+
""".strip()
        )

    def test_custom_format_multi_type(self) -> None:
        table = PrettyTable(["col_date", "col_str", "col_float", "col_int"])
        table.add_row([dt.date(2021, 1, 1), "January", 12345.12345, 12345678])
        table.add_row([dt.date(2021, 2, 1), "February", 54321.12345, 87654321])
        table.custom_format["col_date"] = lambda f, v: v.strftime("%d %b %Y")
        table.custom_format["col_float"] = lambda f, v: f"{v:.3f}"
        table.custom_format["col_int"] = lambda f, v: f"{v:,}"
        assert (
            table.get_string().strip()
            == """
+-------------+----------+-----------+------------+
|   col_date  | col_str  | col_float |  col_int   |
+-------------+----------+-----------+------------+
| 01 Jan 2021 | January  | 12345.123 | 12,345,678 |
| 01 Feb 2021 | February | 54321.123 | 87,654,321 |
+-------------+----------+-----------+------------+
""".strip()
        )

    def test_custom_format_multi_type_using_on_function(self) -> None:
        table = PrettyTable(["col_date", "col_str", "col_float", "col_int"])
        table.add_row([dt.date(2021, 1, 1), "January", 12345.12345, 12345678])
        table.add_row([dt.date(2021, 2, 1), "February", 54321.12345, 87654321])

        def my_format(col: str, value: Any) -> str:
            if col == "col_date":
                return value.strftime("%d %b %Y")
            if col == "col_float":
                return f"{value:.3f}"
            if col == "col_int":
                return f"{value:,}"
            return str(value)

        table.custom_format = my_format
        assert (
            table.get_string().strip()
            == """
+-------------+----------+-----------+------------+
|   col_date  | col_str  | col_float |  col_int   |
+-------------+----------+-----------+------------+
| 01 Jan 2021 | January  | 12345.123 | 12,345,678 |
| 01 Feb 2021 | February | 54321.123 | 87,654,321 |
+-------------+----------+-----------+------------+
""".strip()
        )


class TestRepr:
    def test_default_repr(self, row_prettytable: PrettyTable) -> None:
        assert row_prettytable.__str__() == row_prettytable.__repr__()

    def test_jupyter_repr(self, row_prettytable: PrettyTable) -> None:
        assert row_prettytable._repr_html_() == row_prettytable.get_html_string()


class TestBreakOnHyphens:
    row = [
        "bluedevil breeze breeze-gtk eos-bash-shared glib2 "
        "kactivitymanagerd kde-cli-tools kde-gtk-config kdecoration"
    ]
    EXPECTED_TRUE = """+------------------------------------------+
|                 Field 1                  |
+------------------------------------------+
|  bluedevil breeze breeze-gtk eos-bash-   |
| shared glib2 kactivitymanagerd kde-cli-  |
|     tools kde-gtk-config kdecoration     |
+------------------------------------------+"""
    EXPECTED_FALSE = """+------------------------------------------+
|                 Field 1                  |
+------------------------------------------+
|       bluedevil breeze breeze-gtk        |
| eos-bash-shared glib2 kactivitymanagerd  |
| kde-cli-tools kde-gtk-config kdecoration |
+------------------------------------------+"""

    def test_break_on_hyphens(self) -> None:
        table = PrettyTable(max_width=40)
        table.break_on_hyphens = False
        assert not table.break_on_hyphens
        table.add_row(self.row)
        assert table.get_string().strip() == self.EXPECTED_FALSE

    def test_break_on_hyphens_on_init(self) -> None:
        table = PrettyTable(max_width=40, break_on_hyphens=False)
        assert not table._break_on_hyphens
        assert not table.break_on_hyphens
        table.add_row(self.row)
        assert table.get_string().strip() == self.EXPECTED_FALSE

    def test_break_on_hyphens_default(self) -> None:
        table = PrettyTable(max_width=40)
        assert table.break_on_hyphens
        table.add_row(self.row)
        assert table.get_string().strip() == self.EXPECTED_TRUE


class TestWidth:
    colored = "\033[31mC\033[32mO\033[31mL\033[32mO\033[31mR\033[32mE\033[31mD\033[0m"

    def test_color(self) -> None:
        table = PrettyTable(["Field 1", "Field 2"])
        table.add_row([self.colored, self.colored])
        table.add_row(["nothing", "neither"])
        result = table.get_string()
        assert (
            result.strip()
            == f"""
+---------+---------+
| Field 1 | Field 2 |
+---------+---------+
| {self.colored} | {self.colored} |
| nothing | neither |
+---------+---------+
""".strip()
        )

    def test_reset(self) -> None:
        table = PrettyTable(["Field 1", "Field 2"])
        table.add_row(["abc def\033(B", "\033[31mabc def\033[m"])
        table.add_row(["nothing", "neither"])
        result = table.get_string()
        assert (
            result.strip()
            == """
+---------+---------+
| Field 1 | Field 2 |
+---------+---------+
| abc def\033(B | \033[31mabc def\033[m |
| nothing | neither |
+---------+---------+
""".strip()
        )

    @pytest.mark.parametrize(
        "loops, fields, desired_width, border, internal_border",
        [
            (15, ["Test table"], 20, True, False),
            (16, ["Test table"], 21, True, False),
            (18, ["Test table", "Test table 2"], 40, True, False),
            (19, ["Test table", "Test table 2"], 41, True, False),
            (21, ["Test table", "Test col 2", "Test col 3"], 50, True, False),
            (22, ["Test table", "Test col 2", "Test col 3"], 51, True, False),
            (19, ["Test table"], 20, False, False),
            (20, ["Test table"], 21, False, False),
            (25, ["Test table", "Test table 2"], 40, False, False),
            (26, ["Test table", "Test table 2"], 41, False, False),
            (25, ["Test table", "Test col 2", "Test col 3"], 50, False, False),
            (26, ["Test table", "Test col 2", "Test col 3"], 51, False, False),
            (18, ["Test table"], 20, False, True),
            (19, ["Test table"], 21, False, True),
            (23, ["Test table", "Test table 2"], 40, False, True),
            (24, ["Test table", "Test table 2"], 41, False, True),
            (22, ["Test table", "Test col 2", "Test col 3"], 50, False, True),
            (23, ["Test table", "Test col 2", "Test col 3"], 51, False, True),
        ],
    )
    def test_min_table_width(
        self,
        loops: int,
        fields: list[str],
        desired_width: int,
        border: bool,
        internal_border: bool,
    ) -> None:
        for col_width in range(loops):
            x = prettytable.PrettyTable()
            x.border = border
            x.preserve_internal_border = internal_border
            x.field_names = fields
            x.add_row(["X" * col_width] + ["" for _ in range(len(fields) - 1)])
            x.min_table_width = desired_width
            t = x.get_string()
            if border is False and internal_border is False:
                assert [len(x) for x in t.split("\n")] == [desired_width, desired_width]
            elif border is False and internal_border is True:
                assert [len(x) for x in t.split("\n")] == [
                    desired_width,
                    desired_width - 1,
                    desired_width,
                ]
            else:
                assert [len(x) for x in t.split("\n")] == [
                    desired_width,
                    desired_width,
                    desired_width,
                    desired_width,
                    desired_width,
                ]

    def test_max_table_width(self) -> None:
        table = PrettyTable()
        table.max_table_width = 5
        table.add_row([0])

        # FIXME: Table is wider than table.max_table_width
        assert (
            table.get_string().strip()
            == """
+----+
| Fi |
+----+
| 0  |
+----+
""".strip()
        )

    def test_max_table_width_wide(self) -> None:
        table = PrettyTable()
        table.max_table_width = 52
        table.add_row(
            [
                0,
                0,
                0,
                0,
                0,
                "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam "
                "nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam "
                "erat, sed diam voluptua",
            ]
        )

        assert (
            table.get_string().strip()
            == """
+---+---+---+---+---+------------------------------+
| F | F | F | F | F |           Field 6            |
+---+---+---+---+---+------------------------------+
| 0 | 0 | 0 | 0 | 0 | Lorem ipsum dolor sit amet,  |
|   |   |   |   |   | consetetur sadipscing elitr, |
|   |   |   |   |   |    sed diam nonumy eirmod    |
|   |   |   |   |   | tempor invidunt ut labore et |
|   |   |   |   |   | dolore magna aliquyam erat,  |
|   |   |   |   |   |      sed diam voluptua       |
+---+---+---+---+---+------------------------------+""".strip()
        )

    def test_max_table_width_wide2(self) -> None:
        table = PrettyTable()
        table.max_table_width = 70
        table.add_row(
            [
                "Lorem",
                "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam ",
                "ipsum",
                "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam ",
                "dolor",
                "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam ",
            ]
        )

        assert (
            table.get_string().strip()
            == """
+---+-----------------+---+-----------------+---+-----------------+
| F |     Field 2     | F |     Field 4     | F |     Field 6     |
+---+-----------------+---+-----------------+---+-----------------+
| L |   Lorem ipsum   | i |   Lorem ipsum   | d |   Lorem ipsum   |
| o | dolor sit amet, | p | dolor sit amet, | o | dolor sit amet, |
| r |    consetetur   | s |    consetetur   | l |    consetetur   |
| e |    sadipscing   | u |    sadipscing   | o |    sadipscing   |
| m | elitr, sed diam | m | elitr, sed diam | r | elitr, sed diam |
+---+-----------------+---+-----------------+---+-----------------+""".strip()
        )

    @pytest.mark.parametrize("set_width_parameter", [True, False])
    def test_table_max_width_wo_header_width(self, set_width_parameter: bool) -> None:
        headers = [
            "A Field Name",
            "B Field Name",
            "D Field Name",
            "E Field Name",
            "F Field Name",
            "G Field Name",
            "H Field Name",
            "I Field Name",
            "J Field Name",
            "K Field Name",
            "L Field Name",
            "M Field Name",
        ]
        row = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]
        expected = """+---+---+---+---+---+---+---+---+---+---+----+----+
| A | B | D | E | F | G | H | I | J | K | L  | M  |
+---+---+---+---+---+---+---+---+---+---+----+----+
| 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | 10 | 11 |
+---+---+---+---+---+---+---+---+---+---+----+----+"""

        if set_width_parameter:
            table = PrettyTable(headers, use_header_width=False)
        else:
            table = PrettyTable(headers)
            table.use_header_width = False
        table.add_row(row)

        assert table.get_string() == expected

    def test_table_width_on_init_wo_columns(self) -> None:
        """See also #272"""
        table = PrettyTable(max_width=10)
        table.add_row(
            [
                "Lorem",
                "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam ",
                "ipsum",
                "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam ",
                "dolor",
                "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam ",
            ]
        )

        assert (
            table.get_string().strip()
            == """
+---------+------------+---------+------------+---------+------------+
| Field 1 |  Field 2   | Field 3 |  Field 4   | Field 5 |  Field 6   |
+---------+------------+---------+------------+---------+------------+
|  Lorem  |   Lorem    |  ipsum  |   Lorem    |  dolor  |   Lorem    |
|         |   ipsum    |         |   ipsum    |         |   ipsum    |
|         | dolor sit  |         | dolor sit  |         | dolor sit  |
|         |   amet,    |         |   amet,    |         |   amet,    |
|         | consetetur |         | consetetur |         | consetetur |
|         | sadipscing |         | sadipscing |         | sadipscing |
|         | elitr, sed |         | elitr, sed |         | elitr, sed |
|         |    diam    |         |    diam    |         |    diam    |
+---------+------------+---------+------------+---------+------------+""".strip()
        )

    def test_table_width_on_init_with_columns(self) -> None:
        """See also #272"""
        table = PrettyTable(
            ["Field 1", "Field 2", "Field 3", "Field 4", "Field 5", "Field 6"],
            max_width=10,
        )
        table.add_row(
            [
                "Lorem",
                "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam ",
                "ipsum",
                "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam ",
                "dolor",
                "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam ",
            ]
        )

        assert (
            table.get_string().strip()
            == """
+---------+------------+---------+------------+---------+------------+
| Field 1 |  Field 2   | Field 3 |  Field 4   | Field 5 |  Field 6   |
+---------+------------+---------+------------+---------+------------+
|  Lorem  |   Lorem    |  ipsum  |   Lorem    |  dolor  |   Lorem    |
|         |   ipsum    |         |   ipsum    |         |   ipsum    |
|         | dolor sit  |         | dolor sit  |         | dolor sit  |
|         |   amet,    |         |   amet,    |         |   amet,    |
|         | consetetur |         | consetetur |         | consetetur |
|         | sadipscing |         | sadipscing |         | sadipscing |
|         | elitr, sed |         | elitr, sed |         | elitr, sed |
|         |    diam    |         |    diam    |         |    diam    |
+---------+------------+---------+------------+---------+------------+""".strip()
        )

    def test_table_minwidth_on_init_with_columns(self) -> None:
        table = PrettyTable(["Field 1", "Field 2"], min_width=20)
        table.add_row(
            [
                "Lorem",
                "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam ",
            ]
        )

        assert (
            table.get_string().strip()
            == """+----------------------+--------------------------------------------------------------------+
|       Field 1        |                              Field 2                               |
+----------------------+--------------------------------------------------------------------+
|        Lorem         | Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam  |
+----------------------+--------------------------------------------------------------------+"""  # noqa: E501
        )

    def test_table_min_max_width_on_init_with_columns(self) -> None:
        table = PrettyTable(["Field 1", "Field 2"], min_width=20, max_width=40)
        table.add_row(
            [
                "Lorem",
                "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam ",
            ]
        )

        assert (
            table.get_string().strip()
            == """+----------------------+------------------------------------------+
|       Field 1        |                 Field 2                  |
+----------------------+------------------------------------------+
|        Lorem         |  Lorem ipsum dolor sit amet, consetetur  |
|                      |        sadipscing elitr, sed diam        |
+----------------------+------------------------------------------+"""
        )

    def test_table_float_formatting_on_init_wo_columns(self) -> None:
        """See also #243"""
        table = prettytable.PrettyTable(float_format="10.2")
        table.field_names = ["Metric", "Initial sol.", "Best sol."]
        table.add_rows([["foo", 1.0 / 3.0, 1.0 / 3.0]])

        assert (
            table.get_string().strip()
            == """
+--------+--------------+------------+
| Metric | Initial sol. | Best sol.  |
+--------+--------------+------------+
|  foo   |        0.33  |       0.33 |
+--------+--------------+------------+""".strip()
        )

    def test_max_table_width_wide_vrules_frame(self) -> None:
        table = PrettyTable()
        table.max_table_width = 52
        table.vrules = VRuleStyle.FRAME
        table.add_row(
            [
                0,
                0,
                0,
                0,
                0,
                "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam "
                "nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam "
                "erat, sed diam voluptua",
            ]
        )

        assert (
            table.get_string().strip()
            == """
+--------------------------------------------------+
| F   F   F   F   F             Field 6            |
+--------------------------------------------------+
| 0   0   0   0   0   Lorem ipsum dolor sit amet,  |
|                     consetetur sadipscing elitr, |
|                        sed diam nonumy eirmod    |
|                     tempor invidunt ut labore et |
|                     dolore magna aliquyam erat,  |
|                          sed diam voluptua       |
+--------------------------------------------------+""".strip()
        )

    def test_max_table_width_wide_vrules_none(self) -> None:
        table = PrettyTable()
        table.max_table_width = 52
        table.vrules = VRuleStyle.NONE
        table.add_row(
            [
                0,
                0,
                0,
                0,
                0,
                "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam "
                "nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam "
                "erat, sed diam voluptua",
            ]
        )

        assert (
            table.get_string().strip()
            == """
----------------------------------------------------
  F   F   F   F   F             Field 6             
----------------------------------------------------
  0   0   0   0   0   Lorem ipsum dolor sit amet,   
                      consetetur sadipscing elitr,  
                         sed diam nonumy eirmod     
                      tempor invidunt ut labore et  
                      dolore magna aliquyam erat,   
                           sed diam voluptua        
----------------------------------------------------""".strip()  # noqa: W291
        )


class TestFields:
    def test_fields_at_class_declaration(self) -> None:
        table = PrettyTable(
            field_names=CITY_DATA_HEADER,
            fields=["City name", "Annual Rainfall"],
        )
        for row in CITY_DATA:
            table.add_row(row)
        assert (
            """+-----------+-----------------+
| City name | Annual Rainfall |
+-----------+-----------------+
|  Adelaide |      600.5      |
|  Brisbane |      1146.4     |
|   Darwin  |      1714.7     |
|   Hobart  |      619.5      |
|   Sydney  |      1214.8     |
| Melbourne |      646.9      |
|   Perth   |      869.4      |
+-----------+-----------------+"""
            == table.get_string().strip()
        )

    def test_fields(self) -> None:
        table = PrettyTable()
        table.field_names = CITY_DATA_HEADER
        table.fields = ["City name", "Annual Rainfall"]
        for row in CITY_DATA:
            table.add_row(row)
        assert (
            """+-----------+-----------------+
| City name | Annual Rainfall |
+-----------+-----------------+
|  Adelaide |      600.5      |
|  Brisbane |      1146.4     |
|   Darwin  |      1714.7     |
|   Hobart  |      619.5      |
|   Sydney  |      1214.8     |
| Melbourne |      646.9      |
|   Perth   |      869.4      |
+-----------+-----------------+"""
            == table.get_string().strip()
        )


class TestGeneralOutput:
    def test_copy(self, helper_table: PrettyTable) -> None:
        t_copy = helper_table.copy()
        assert helper_table.get_string() == t_copy.get_string()

    def test_text(self, helper_table: PrettyTable) -> None:
        assert helper_table.get_formatted_string("text") == helper_table.get_string()
        # test with default arg, too
        assert helper_table.get_formatted_string() == helper_table.get_string()
        # args passed through
        assert helper_table.get_formatted_string(
            border=False
        ) == helper_table.get_string(border=False)

    def test_csv(self, helper_table: PrettyTable) -> None:
        assert helper_table.get_formatted_string("csv") == helper_table.get_csv_string()
        # args passed through
        assert helper_table.get_formatted_string(
            "csv", border=False
        ) == helper_table.get_csv_string(border=False)

    def test_json(self, helper_table: PrettyTable) -> None:
        assert (
            helper_table.get_formatted_string("json") == helper_table.get_json_string()
        )
        # args passed through
        assert helper_table.get_formatted_string(
            "json", border=False
        ) == helper_table.get_json_string(border=False)

    def test_html(self, helper_table: PrettyTable) -> None:
        assert (
            helper_table.get_formatted_string("html") == helper_table.get_html_string()
        )
        # args passed through
        assert helper_table.get_formatted_string(
            "html", border=False
        ) == helper_table.get_html_string(border=False)

    def test_latex(self, helper_table: PrettyTable) -> None:
        assert (
            helper_table.get_formatted_string("latex")
            == helper_table.get_latex_string()
        )
        # args passed through
        assert helper_table.get_formatted_string(
            "latex", border=False
        ) == helper_table.get_latex_string(border=False)

    def test_mediawiki(self, helper_table: PrettyTable) -> None:
        assert helper_table.get_formatted_string(
            "mediawiki", border=False
        ) == helper_table.get_mediawiki_string(border=False)

    def test_invalid(self, helper_table: PrettyTable) -> None:
        with pytest.raises(ValueError):
            helper_table.get_formatted_string("pdf")


class TestDeprecations:
    @pytest.mark.parametrize(
        "module_name",
        [
            "prettytable",
            "prettytable.prettytable",
        ],
    )
    @pytest.mark.parametrize(
        "name",
        [
            "FRAME",
            "ALL",
            "NONE",
            "HEADER",
        ],
    )
    def test_hrule_constant_deprecations(self, module_name: str, name: str) -> None:
        with pytest.deprecated_call(match=f"the '{name}' constant is deprecated"):
            exec(f"from {module_name} import {name}")

    @pytest.mark.parametrize(
        "module_name",
        [
            "prettytable",
            "prettytable.prettytable",
        ],
    )
    @pytest.mark.parametrize(
        "name",
        [
            "DEFAULT",
            "MSWORD_FRIENDLY",
            "PLAIN_COLUMNS",
            "MARKDOWN",
            "ORGMODE",
            "DOUBLE_BORDER",
            "SINGLE_BORDER",
            "RANDOM",
        ],
    )
    def test_table_style_constant_deprecations(
        self, module_name: str, name: str
    ) -> None:
        with pytest.deprecated_call(match=f"the '{name}' constant is deprecated"):
            exec(f"from {module_name} import {name}")



from __future__ import annotations

import io

import pytest
from test_prettytable import CITY_DATA, CITY_DATA_HEADER

from prettytable import PrettyTable, from_csv, from_mediawiki


@pytest.fixture(scope="function")
def col_prettytable() -> PrettyTable:
    # Column by column...
    table = PrettyTable()
    for idx, colname in enumerate(CITY_DATA_HEADER):
        table.add_column(colname, [row[idx] for row in CITY_DATA])
    return table


@pytest.fixture(scope="function")
def city_data() -> PrettyTable:
    """Just build the Australian capital city data example table."""
    table = PrettyTable(CITY_DATA_HEADER)
    for row in CITY_DATA:
        table.add_row(row)
    return table


@pytest.fixture(scope="function")
def city_data_from_csv() -> PrettyTable:
    csv_string = ", ".join(CITY_DATA_HEADER)
    for row in CITY_DATA:
        csv_string += "\n" + ",".join(str(fld) for fld in row)
    csv_fp = io.StringIO(csv_string)
    return from_csv(csv_fp)


@pytest.fixture(scope="function")
def city_data_from_mediawiki() -> PrettyTable:
    wiki_text = '{| class="wikitable"\n'
    wiki_text += "|-\n"
    wiki_text += "! " + " !! ".join(CITY_DATA_HEADER) + "\n"
    for row in CITY_DATA:
        wiki_text += "|-\n"
        wiki_text += "| " + " || ".join(str(item) for item in row) + "\n"
    wiki_text += "|}"
    return from_mediawiki(wiki_text)


@pytest.fixture
def mix_prettytable() -> PrettyTable:
    # A mix of both!
    table = PrettyTable()
    table.field_names = [CITY_DATA_HEADER[0], CITY_DATA_HEADER[1]]
    for row in CITY_DATA:
        table.add_row([row[0], row[1]])
    for idx, colname in enumerate(CITY_DATA_HEADER[2:]):
        table.add_column(colname, [row[idx + 2] for row in CITY_DATA])
    return table


@pytest.fixture(scope="function")
def field_name_less_table() -> PrettyTable:
    table = PrettyTable()
    for row in CITY_DATA:
        table.add_row(row)
    return table


@pytest.fixture(scope="function")
def row_prettytable(field_name_less_table: PrettyTable) -> PrettyTable:
    # Row by row...
    field_name_less_table.field_names = CITY_DATA_HEADER
    return field_name_less_table


@pytest.fixture(scope="function")
def empty_helper_table() -> PrettyTable:
    return PrettyTable(["", "Field 1", "Field 2", "Field 3"])


@pytest.fixture(scope="function")
def helper_table(empty_helper_table: PrettyTable) -> PrettyTable:
    v = 1
    for row in range(3):
        # Some have spaces, some not, to help test padding columns of different widths
        empty_helper_table.add_row([v, f"value {v}", f"value{v+1}", f"value{v+2}"])
        v += 3
    return empty_helper_table



from __future__ import annotations

from test_prettytable import CITY_DATA, CITY_DATA_HEADER

from prettytable import PrettyTable, RowType


class TestSorting:
    def test_sort_by_different_per_columns(self, city_data: PrettyTable) -> None:
        city_data.sortby = city_data.field_names[0]
        old = city_data.get_string()
        for field in city_data.field_names[1:]:
            city_data.sortby = field
            new = city_data.get_string()
            assert new != old

    def test_reverse_sort(self, city_data: PrettyTable) -> None:
        for field in city_data.field_names:
            city_data.sortby = field
            city_data.reversesort = False
            forward = city_data.get_string()
            city_data.reversesort = True
            backward = city_data.get_string()
            forward_lines = forward.split("\n")[2:]  # Discard header lines
            backward_lines = backward.split("\n")[2:]
            backward_lines.reverse()
            assert forward_lines == backward_lines

    def test_sort_key(self, city_data: PrettyTable) -> None:
        # Test sorting by length of city name
        def key(vals: RowType) -> list[int]:
            vals[0] = len(vals[0])
            return vals

        city_data.sortby = "City name"
        city_data.sort_key = key
        assert (
            city_data.get_string().strip()
            == """
+-----------+------+------------+-----------------+
| City name | Area | Population | Annual Rainfall |
+-----------+------+------------+-----------------+
|   Perth   | 5386 |  1554769   |      869.4      |
|   Darwin  | 112  |   120900   |      1714.7     |
|   Hobart  | 1357 |   205556   |      619.5      |
|   Sydney  | 2058 |  4336374   |      1214.8     |
|  Adelaide | 1295 |  1158259   |      600.5      |
|  Brisbane | 5905 |  1857594   |      1146.4     |
| Melbourne | 1566 |  3806092   |      646.9      |
+-----------+------+------------+-----------------+
""".strip()
        )

    def test_sort_key_at_class_declaration(self) -> None:
        # Test sorting by length of city name
        def key(vals: RowType) -> list[int]:
            vals[0] = len(vals[0])
            return vals

        table = PrettyTable(
            field_names=CITY_DATA_HEADER,
            sortby="City name",
            sort_key=key,
        )
        assert table.sort_key == key
        for row in CITY_DATA:
            table.add_row(row)
        assert (
            """+-----------+------+------------+-----------------+
| City name | Area | Population | Annual Rainfall |
+-----------+------+------------+-----------------+
|   Perth   | 5386 |  1554769   |      869.4      |
|   Darwin  | 112  |   120900   |      1714.7     |
|   Hobart  | 1357 |   205556   |      619.5      |
|   Sydney  | 2058 |  4336374   |      1214.8     |
|  Adelaide | 1295 |  1158259   |      600.5      |
|  Brisbane | 5905 |  1857594   |      1146.4     |
| Melbourne | 1566 |  3806092   |      646.9      |
+-----------+------+------------+-----------------+"""
            == table.get_string().strip()
        )

    def test_sort_slice(self) -> None:
        """Make sure sorting and slicing interact in the expected way"""
        table = PrettyTable(["Foo"])
        for i in range(20, 0, -1):
            table.add_row([i])
        new_style = table.get_string(sortby="Foo", end=10)
        assert "10" in new_style
        assert "20" not in new_style
        oldstyle = table.get_string(sortby="Foo", end=10, oldsortslice=True)
        assert "10" not in oldstyle
        assert "20" in oldstyle

    def test_sortby_at_class_declaration(self) -> None:
        """
        Fix #354 where initialization of a table with sortby fails
        """
        table = PrettyTable(
            field_names=CITY_DATA_HEADER,
            sortby="Area",
        )
        assert table.sortby == "Area"
        for row in CITY_DATA:
            table.add_row(row)
        assert (
            """+-----------+------+------------+-----------------+
| City name | Area | Population | Annual Rainfall |
+-----------+------+------------+-----------------+
|   Darwin  | 112  |   120900   |      1714.7     |
|  Adelaide | 1295 |  1158259   |      600.5      |
|   Hobart  | 1357 |   205556   |      619.5      |
| Melbourne | 1566 |  3806092   |      646.9      |
|   Sydney  | 2058 |  4336374   |      1214.8     |
|   Perth   | 5386 |  1554769   |      869.4      |
|  Brisbane | 5905 |  1857594   |      1146.4     |
+-----------+------+------------+-----------------+"""
            == table.get_string().strip()
        )



from __future__ import annotations

import pytest
from test_prettytable import CITY_DATA, CITY_DATA_HEADER

from prettytable.colortable import RESET_CODE, ColorTable, Theme, Themes

TYPE_CHECKING = False
if TYPE_CHECKING:
    from prettytable import PrettyTable


@pytest.fixture
def row_colortable() -> PrettyTable:
    table = ColorTable()
    table.field_names = CITY_DATA_HEADER
    for row in CITY_DATA:
        table.add_row(row)
    return table


@pytest.fixture
def color_theme() -> Theme:
    return Theme(
        default_color="31",
        vertical_color="32",
        horizontal_color="33",
        junction_color="34",
    )


class TestColorTable:
    def test_themeless(
        self, row_prettytable: PrettyTable, row_colortable: ColorTable
    ) -> None:
        # Not worth the logic customizing the reset code
        # For now we'll just get rid of it
        assert (
            row_colortable.get_string().replace(RESET_CODE, "")
            == row_prettytable.get_string()
        )

    def test_theme_setter(self, color_theme: Theme) -> None:
        table1 = ColorTable(theme=color_theme)

        table2 = ColorTable()
        table2.theme = color_theme

        assert table1.theme == table2.theme

        dict1 = table1.__dict__
        dict2 = table2.__dict__

        # So we don't compare functions
        for func in ("_sort_key", "_row_filter"):
            del dict1[func]
            del dict2[func]

        assert dict1 == dict2


class TestFormatCode:
    def test_basic(self) -> None:
        assert Theme.format_code("31") == "\x1b[31m"

    def test_prefix(self) -> None:
        assert Theme.format_code("\x1b[35m") == "\x1b[35m"

    def test_escapes(self) -> None:
        assert Theme.format_code("\033[41m") == "\x1b[41m"
        assert Theme.format_code("\u001b[41m") == "\x1b[41m"

    def test_empty(self) -> None:
        assert Theme.format_code("") == ""

    def test_stripped(self) -> None:
        assert Theme.format_code("\t\t     \t") == ""

    def test_multiple(self) -> None:
        assert Theme.format_code("30;42") == "\x1b[30;42m"
        assert Theme.format_code("\x1b[30;42m") == "\x1b[30;42m"


class TestColorTableRendering:
    """Tests for the rendering of the color table

    Methods
    -------
    test_color_table_rendering
        Tests the color table rendering using the default alignment (`'c'`)
    """

    @pytest.mark.parametrize(
        ["with_title", "with_header"],
        [
            (False, True),  # the default
            (True, True),  # titled
            (True, False),  # titled, no header
            (True, True),  # both title and header
        ],
    )
    def test_color_table_rendering(self, with_title: bool, with_header: bool) -> None:
        """Tests the color table rendering using the default alignment (`'c'`)"""
        chars = {
            "+": "\x1b[36m+\x1b[0m\x1b[96m",
            "-": "\x1b[34m-\x1b[0m\x1b[96m",
            "|": "\x1b[34m|\x1b[0m\x1b[96m",
            " ": " ",
        }

        plus = chars.get("+")
        minus = chars.get("-")
        pipe = chars.get("|")
        space = chars.get(" ")
        assert isinstance(plus, str)
        assert isinstance(minus, str)
        assert isinstance(pipe, str)
        assert isinstance(space, str)

        # +-----------------------+
        # |        Efforts        |
        # +---+---+---+---+---+---+
        # | A | B | C | D | E | f |
        # +---+---+---+---+---+---+
        # | 1 | 2 | 3 | 4 | 5 | 6 |
        # +---+---+---+---+---+---+

        header = (
            plus + minus * 23 + plus,
            pipe + space * 8 + "Efforts" + space * 8 + pipe,
            (plus + minus * 3) * 6 + plus,
        )

        body = (
            "".join(pipe + space + char + space for char in "ABCDEF") + pipe,
            (plus + minus * 3) * 6 + plus,
            "".join(pipe + space + char + space for char in "123456") + pipe,
            (plus + minus * 3) * 6 + plus,
        )

        if with_title:
            header_str = str("\n".join(header))
        else:
            header_str = str(header[2])
        if with_header:
            body_str = str("\n".join(body))
        else:
            body_str = str("\n".join(body[2:]))

        table = ColorTable(
            ("A", "B", "C", "D", "E", "F"),
            theme=Themes.OCEAN,
        )

        if with_title:
            table.title = "Efforts"
        table.header = with_header
        table.add_row([1, 2, 3, 4, 5, 6])

        expected = header_str + "\n" + body_str + "\x1b[0m"
        result = str(table)

        assert expected == result

    def test_all_themes(self) -> None:
        """Tests rendering with all available themes"""
        table = ColorTable(
            ("A", "B", "C", "D", "E", "F"),
        )
        table.title = "Theme Test"
        table.add_row([1, 2, 3, 4, 5, 6])

        for theme_name, theme in vars(Themes).items():
            if isinstance(theme, Theme):
                table.theme = theme
                result = str(table)
                assert result  # Simple check to ensure rendering doesn't fail



from __future__ import annotations

import pytest

from prettytable import PrettyTable, from_mediawiki


class TestMediaWikiOutput:
    def test_mediawiki_output(self, helper_table: PrettyTable) -> None:
        assert (
            helper_table.get_mediawiki_string(header=True).strip()
            == """
{| class="wikitable"
|-
!  !! Field 1 !! Field 2 !! Field 3
|-
| 1 || value 1 || value2 || value3
|-
| 4 || value 4 || value5 || value6
|-
| 7 || value 7 || value8 || value9
|}
""".strip()
        )

    def test_mediawiki_output_without_header(self, helper_table: PrettyTable) -> None:
        assert (
            helper_table.get_mediawiki_string(header=False).strip()
            == """
{| class="wikitable"
|-
| 1 || value 1 || value2 || value3
|-
| 4 || value 4 || value5 || value6
|-
| 7 || value 7 || value8 || value9
|}
""".strip()
        )

    def test_mediawiki_output_with_caption(self, helper_table: PrettyTable) -> None:
        assert (
            helper_table.get_mediawiki_string(
                title="Optional caption", header=True
            ).strip()
            == """
{| class="wikitable"
|+ Optional caption
|-
!  !! Field 1 !! Field 2 !! Field 3
|-
| 1 || value 1 || value2 || value3
|-
| 4 || value 4 || value5 || value6
|-
| 7 || value 7 || value8 || value9
|}
""".strip()
        )

    def test_mediawiki_output_with_attributes(self, helper_table: PrettyTable) -> None:
        assert (
            helper_table.get_mediawiki_string(
                attributes={"class": "mytable", "id": "table1"}, header=True
            ).strip()
            == """
{| class="mytable" id="table1"
|-
!  !! Field 1 !! Field 2 !! Field 3
|-
| 1 || value 1 || value2 || value3
|-
| 4 || value 4 || value5 || value6
|-
| 7 || value 7 || value8 || value9
|}
            """.strip()
        )

    def test_mediawiki_output_with_fields_option(
        self, helper_table: PrettyTable
    ) -> None:
        assert (
            helper_table.get_mediawiki_string(
                fields=["Field 1", "Field 3"], header=True
            ).strip()
            == """
{| class="wikitable"
|-
! Field 1 !! Field 3
|-
| value 1 || value3
|-
| value 4 || value6
|-
| value 7 || value9
|}
            """.strip()
        )


class TestMediaWikiConstructor:
    def test_mediawiki_and_back(self, city_data: PrettyTable) -> None:
        mediawiki_string = city_data.get_mediawiki_string()
        new_table = from_mediawiki(mediawiki_string)
        assert new_table.get_string() == city_data.get_string()

    def test_from_mediawiki_ignores_non_table_text(
        self, city_data: PrettyTable
    ) -> None:
        wiki_table = city_data.get_mediawiki_string()
        wiki_text = f"Before table text.\n{wiki_table}\nAfter table text."
        table = from_mediawiki(wiki_text)
        output = table.get_string()

        for header in city_data.field_names:
            assert header in output
        for row in city_data._rows:
            for cell in row:
                assert str(cell) in output
        assert "Before table text." not in output
        assert "After table text." not in output

    def test_from_mediawiki_ignores_caption(self) -> None:
        wiki_text = """
{| class="wikitable"
|+ Optional caption
|-
! Field 1 !! Field 2
|-
| value 1 || value2
|-
| value 4 || value5
|}
    """
        table = from_mediawiki(wiki_text)
        output = table.get_string()
        assert "Optional caption" not in output
        assert "value 1" in output

    def test_from_mediawiki_no_header(self) -> None:
        wiki_text = """
{| class="wikitable"
|-
| value 1 || value2
|-
| value 4 || value5
|-
| value 7 || value8
|}
    """
        with pytest.raises(
            ValueError, match="No valid header found in the MediaWiki table."
        ):
            from_mediawiki(wiki_text)

    def test_from_mediawiki_row_length_mismatch(self) -> None:
        wiki_text = """
{| class="wikitable"
|-
! Field 1 !! Field 2
|-
| value 1 || value2 || value3
|-
| value 4 || value5 || value6
|}
    """
        with pytest.raises(
            ValueError, match="Row length mismatch between header and body."
        ):
            from_mediawiki(wiki_text)



from __future__ import annotations

import pytest

from prettytable import HRuleStyle, PrettyTable, from_html, from_html_one


class TestHtmlConstructor:
    def test_html_and_back(self, city_data: PrettyTable) -> None:
        html_string = city_data.get_html_string()
        new_table = from_html(html_string)[0]
        assert new_table.get_string() == city_data.get_string()

    def test_html_one_and_back(self, city_data: PrettyTable) -> None:
        html_string = city_data.get_html_string()
        new_table = from_html_one(html_string)
        assert new_table.get_string() == city_data.get_string()

    def test_html_one_fail_on_many(self, city_data: PrettyTable) -> None:
        html_string = city_data.get_html_string()
        html_string += city_data.get_html_string()
        with pytest.raises(ValueError):
            from_html_one(html_string)


class TestHtmlOutput:
    def test_html_output(self, helper_table: PrettyTable) -> None:
        result = helper_table.get_html_string()
        assert (
            result.strip()
            == """
<table>
    <thead>
        <tr>
            <th></th>
            <th>Field 1</th>
            <th>Field 2</th>
            <th>Field 3</th>
        </tr>
    </thead>
    <tbody>
        <tr>
            <td>1</td>
            <td>value 1</td>
            <td>value2</td>
            <td>value3</td>
        </tr>
        <tr>
            <td>4</td>
            <td>value 4</td>
            <td>value5</td>
            <td>value6</td>
        </tr>
        <tr>
            <td>7</td>
            <td>value 7</td>
            <td>value8</td>
            <td>value9</td>
        </tr>
    </tbody>
</table>
""".strip()
        )

    def test_html_output_formatted(self, helper_table: PrettyTable) -> None:
        result = helper_table.get_html_string(format=True)
        assert (
            result.strip()
            == """
<table frame="box" rules="cols">
    <thead>
        <tr>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center"></th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">Field 1</th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">Field 2</th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">Field 3</th>
        </tr>
    </thead>
    <tbody>
        <tr>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">1</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value 1</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value2</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value3</td>
        </tr>
        <tr>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">4</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value 4</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value5</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value6</td>
        </tr>
        <tr>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">7</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value 7</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value8</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value9</td>
        </tr>
    </tbody>
</table>
""".strip()  # noqa: E501
        )

    def test_html_output_with_title(self, helper_table: PrettyTable) -> None:
        helper_table.title = "Title & Title"
        result = helper_table.get_html_string(
            attributes={"bgcolor": "red", "a<b": "1<2"}
        )
        assert (
            result.strip()
            == """
<table bgcolor="red" a&lt;b="1&lt;2">
    <caption>Title &amp; Title</caption>
    <thead>
        <tr>
            <th></th>
            <th>Field 1</th>
            <th>Field 2</th>
            <th>Field 3</th>
        </tr>
    </thead>
    <tbody>
        <tr>
            <td>1</td>
            <td>value 1</td>
            <td>value2</td>
            <td>value3</td>
        </tr>
        <tr>
            <td>4</td>
            <td>value 4</td>
            <td>value5</td>
            <td>value6</td>
        </tr>
        <tr>
            <td>7</td>
            <td>value 7</td>
            <td>value8</td>
            <td>value9</td>
        </tr>
    </tbody>
</table>
""".strip()
        )

    def test_html_output_formatted_with_title(self, helper_table: PrettyTable) -> None:
        helper_table.title = "Title & Title"
        result = helper_table.get_html_string(
            attributes={"bgcolor": "red", "a<b": "1<2"}, format=True
        )
        assert (
            result.strip()
            == """
<table frame="box" rules="cols" bgcolor="red" a&lt;b="1&lt;2">
    <caption>Title &amp; Title</caption>
    <thead>
        <tr>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center"></th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">Field 1</th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">Field 2</th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">Field 3</th>
        </tr>
    </thead>
    <tbody>
        <tr>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">1</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value 1</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value2</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value3</td>
        </tr>
        <tr>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">4</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value 4</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value5</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value6</td>
        </tr>
        <tr>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">7</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value 7</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value8</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value9</td>
        </tr>
    </tbody>
</table>
""".strip()  # noqa: E501
        )

    def test_html_output_without_escaped_header(
        self, empty_helper_table: PrettyTable
    ) -> None:
        empty_helper_table.field_names = [
            "",
            "Field 1",
            "<em>Field 2</em>",
            "<a href='#'>Field 3</a>",
        ]
        result = empty_helper_table.get_html_string(escape_header=False)
        assert (
            result.strip()
            == """
<table>
    <thead>
        <tr>
            <th></th>
            <th>Field 1</th>
            <th><em>Field 2</em></th>
            <th><a href='#'>Field 3</a></th>
        </tr>
    </thead>
    <tbody>
    </tbody>
</table>
""".strip()
        )

    def test_html_output_without_escaped_data(
        self, empty_helper_table: PrettyTable
    ) -> None:
        empty_helper_table.add_row(
            [
                1,
                "<b>value 1</b>",
                "<span style='text-decoration: underline;'>value2</span>",
                "<a href='#'>value3</a>",
            ]
        )
        result = empty_helper_table.get_html_string(escape_data=False)
        assert (
            result.strip()
            == """
<table>
    <thead>
        <tr>
            <th></th>
            <th>Field 1</th>
            <th>Field 2</th>
            <th>Field 3</th>
        </tr>
    </thead>
    <tbody>
        <tr>
            <td>1</td>
            <td><b>value 1</b></td>
            <td><span style='text-decoration: underline;'>value2</span></td>
            <td><a href='#'>value3</a></td>
        </tr>
    </tbody>
</table>
""".strip()
        )

    def test_html_output_with_escaped_header(
        self, empty_helper_table: PrettyTable
    ) -> None:
        empty_helper_table.field_names = [
            "",
            "Field 1",
            "<em>Field 2</em>",
            "<a href='#'>Field 3</a>",
        ]
        result = empty_helper_table.get_html_string(escape_header=True)
        assert (
            result.strip()
            == """
<table>
    <thead>
        <tr>
            <th></th>
            <th>Field 1</th>
            <th>&lt;em&gt;Field 2&lt;/em&gt;</th>
            <th>&lt;a href=&#x27;#&#x27;&gt;Field 3&lt;/a&gt;</th>
        </tr>
    </thead>
    <tbody>
    </tbody>
</table>
""".strip()
        )

    def test_html_output_with_escaped_data(
        self, empty_helper_table: PrettyTable
    ) -> None:
        empty_helper_table.add_row(
            [
                1,
                "<b>value 1</b>",
                "<span style='text-decoration: underline;'>value2</span>",
                "<a href='#'>value3</a>",
            ]
        )
        result = empty_helper_table.get_html_string(escape_data=True)
        assert (
            result.strip()
            == """
<table>
    <thead>
        <tr>
            <th></th>
            <th>Field 1</th>
            <th>Field 2</th>
            <th>Field 3</th>
        </tr>
    </thead>
    <tbody>
        <tr>
            <td>1</td>
            <td>&lt;b&gt;value 1&lt;/b&gt;</td>
            <td>&lt;span style=&#x27;text-decoration: underline;&#x27;&gt;value2&lt;/span&gt;</td>
            <td>&lt;a href=&#x27;#&#x27;&gt;value3&lt;/a&gt;</td>
        </tr>
    </tbody>
</table>
""".strip()  # noqa: E501
        )

    def test_html_output_formatted_without_escaped_header(
        self, empty_helper_table: PrettyTable
    ) -> None:
        empty_helper_table.field_names = [
            "",
            "Field 1",
            "<em>Field 2</em>",
            "<a href='#'>Field 3</a>",
        ]
        result = empty_helper_table.get_html_string(escape_header=False, format=True)
        assert (
            result.strip()
            == """
<table frame="box" rules="cols">
    <thead>
        <tr>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center"></th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">Field 1</th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center"><em>Field 2</em></th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center"><a href='#'>Field 3</a></th>
        </tr>
    </thead>
    <tbody>
    </tbody>
</table>
""".strip()  # noqa: E501
        )

    def test_html_output_formatted_without_escaped_data(
        self, empty_helper_table: PrettyTable
    ) -> None:
        empty_helper_table.add_row(
            [
                1,
                "<b>value 1</b>",
                "<span style='text-decoration: underline;'>value2</span>",
                "<a href='#'>value3</a>",
            ]
        )
        result = empty_helper_table.get_html_string(escape_data=False, format=True)
        assert (
            result.strip()
            == """
<table frame="box" rules="cols">
    <thead>
        <tr>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center"></th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">Field 1</th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">Field 2</th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">Field 3</th>
        </tr>
    </thead>
    <tbody>
        <tr>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">1</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top"><b>value 1</b></td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top"><span style='text-decoration: underline;'>value2</span></td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top"><a href='#'>value3</a></td>
        </tr>
    </tbody>
</table>
""".strip()  # noqa: E501
        )

    def test_html_output_formatted_with_escaped_header(
        self, empty_helper_table: PrettyTable
    ) -> None:
        empty_helper_table.field_names = [
            "",
            "Field 1",
            "<em>Field 2</em>",
            "<a href='#'>Field 3</a>",
        ]
        result = empty_helper_table.get_html_string(escape_header=True, format=True)
        assert (
            result.strip()
            == """
<table frame="box" rules="cols">
    <thead>
        <tr>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center"></th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">Field 1</th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">&lt;em&gt;Field 2&lt;/em&gt;</th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">&lt;a href=&#x27;#&#x27;&gt;Field 3&lt;/a&gt;</th>
        </tr>
    </thead>
    <tbody>
    </tbody>
</table>
""".strip()  # noqa: E501
        )

    def test_html_output_formatted_with_escaped_data(
        self, empty_helper_table: PrettyTable
    ) -> None:
        empty_helper_table.add_row(
            [
                1,
                "<b>value 1</b>",
                "<span style='text-decoration: underline;'>value2</span>",
                "<a href='#'>value3</a>",
            ]
        )
        result = empty_helper_table.get_html_string(escape_data=True, format=True)
        assert (
            result.strip()
            == """
<table frame="box" rules="cols">
    <thead>
        <tr>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center"></th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">Field 1</th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">Field 2</th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">Field 3</th>
        </tr>
    </thead>
    <tbody>
        <tr>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">1</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">&lt;b&gt;value 1&lt;/b&gt;</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">&lt;span style=&#x27;text-decoration: underline;&#x27;&gt;value2&lt;/span&gt;</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">&lt;a href=&#x27;#&#x27;&gt;value3&lt;/a&gt;</td>
        </tr>
    </tbody>
</table>
""".strip()  # noqa: E501
        )

    def test_table_formatted_html_autoindex(self) -> None:
        """See also #199"""
        table = PrettyTable(["Field 1", "Field 2", "Field 3"])
        for row in range(1, 3 * 3, 3):
            table.add_row(
                [f"value {row*100}", f"value {row+1*100}", f"value {row+2*100}"]
            )
        table.format = True
        table.add_autoindex("I")

        assert (
            table.get_html_string().strip()
            == """
<table frame="box" rules="cols">
    <thead>
        <tr>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">I</th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">Field 1</th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">Field 2</th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">Field 3</th>
        </tr>
    </thead>
    <tbody>
        <tr>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">1</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value 100</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value 101</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value 201</td>
        </tr>
        <tr>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">2</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value 400</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value 104</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value 204</td>
        </tr>
        <tr>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">3</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value 700</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value 107</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value 207</td>
        </tr>
    </tbody>
</table>""".strip()  # noqa: E501
        )

    def test_internal_border_preserved_html(self, helper_table: PrettyTable) -> None:
        helper_table.format = True
        helper_table.border = False
        helper_table.preserve_internal_border = True

        assert (
            helper_table.get_html_string().strip()
            == """
<table rules="cols">
    <thead>
        <tr>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center"></th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">Field 1</th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">Field 2</th>
            <th style="padding-left: 1em; padding-right: 1em; text-align: center">Field 3</th>
        </tr>
    </thead>
    <tbody>
        <tr>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">1</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value 1</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value2</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value3</td>
        </tr>
        <tr>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">4</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value 4</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value5</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value6</td>
        </tr>
        <tr>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">7</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value 7</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value8</td>
            <td style="padding-left: 1em; padding-right: 1em; text-align: center; vertical-align: top">value9</td>
        </tr>
    </tbody>
</table>
""".strip()  # noqa: E501
        )

    def test_break_line_html(self) -> None:
        table = PrettyTable(["Field 1", "Field 2"])
        table.add_row(["value 1", "value2\nsecond line"])
        table.add_row(["value 3", "value4"])
        result = table.get_html_string(hrules=HRuleStyle.ALL)
        assert (
            result.strip()
            == """
<table>
    <thead>
        <tr>
            <th>Field 1</th>
            <th>Field 2</th>
        </tr>
    </thead>
    <tbody>
        <tr>
            <td>value 1</td>
            <td>value2<br>second line</td>
        </tr>
        <tr>
            <td>value 3</td>
            <td>value4</td>
        </tr>
    </tbody>
</table>
""".strip()
        )

    def test_break_line_xhtml(self) -> None:
        table = PrettyTable(["Field 1", "Field 2"])
        table.add_row(["value 1", "value2\nsecond line"])
        table.add_row(["value 3", "value4"])
        result = table.get_html_string(hrules=HRuleStyle.ALL, xhtml=True)
        assert (
            result.strip()
            == """
<table>
    <thead>
        <tr>
            <th>Field 1</th>
            <th>Field 2</th>
        </tr>
    </thead>
    <tbody>
        <tr>
            <td>value 1</td>
            <td>value2<br/>second line</td>
        </tr>
        <tr>
            <td>value 3</td>
            <td>value4</td>
        </tr>
    </tbody>
</table>
""".strip()
        )



from __future__ import annotations

from prettytable import HRuleStyle, PrettyTable, VRuleStyle


class TestLatexOutput:
    def test_latex_output(self, helper_table: PrettyTable) -> None:
        assert helper_table.get_latex_string() == (
            "\\begin{tabular}{cccc}\r\n"
            " & Field 1 & Field 2 & Field 3 \\\\\r\n"
            "1 & value 1 & value2 & value3 \\\\\r\n"
            "4 & value 4 & value5 & value6 \\\\\r\n"
            "7 & value 7 & value8 & value9 \\\\\r\n"
            "\\end{tabular}"
        )
        options = {"fields": ["Field 1", "Field 3"]}
        assert helper_table.get_latex_string(**options) == (
            "\\begin{tabular}{cc}\r\n"
            "Field 1 & Field 3 \\\\\r\n"
            "value 1 & value3 \\\\\r\n"
            "value 4 & value6 \\\\\r\n"
            "value 7 & value9 \\\\\r\n"
            "\\end{tabular}"
        )

    def test_latex_output_formatted(self, helper_table: PrettyTable) -> None:
        assert helper_table.get_latex_string(format=True) == (
            "\\begin{tabular}{|c|c|c|c|}\r\n"
            "\\hline\r\n"
            " & Field 1 & Field 2 & Field 3 \\\\\r\n"
            "1 & value 1 & value2 & value3 \\\\\r\n"
            "4 & value 4 & value5 & value6 \\\\\r\n"
            "7 & value 7 & value8 & value9 \\\\\r\n"
            "\\hline\r\n"
            "\\end{tabular}"
        )

        options = {"fields": ["Field 1", "Field 3"]}
        assert helper_table.get_latex_string(format=True, **options) == (
            "\\begin{tabular}{|c|c|}\r\n"
            "\\hline\r\n"
            "Field 1 & Field 3 \\\\\r\n"
            "value 1 & value3 \\\\\r\n"
            "value 4 & value6 \\\\\r\n"
            "value 7 & value9 \\\\\r\n"
            "\\hline\r\n"
            "\\end{tabular}"
        )

        vrule_options: dict[str, VRuleStyle] = {"vrules": VRuleStyle.FRAME}
        assert helper_table.get_latex_string(format=True, **vrule_options) == (
            "\\begin{tabular}{|cccc|}\r\n"
            "\\hline\r\n"
            " & Field 1 & Field 2 & Field 3 \\\\\r\n"
            "1 & value 1 & value2 & value3 \\\\\r\n"
            "4 & value 4 & value5 & value6 \\\\\r\n"
            "7 & value 7 & value8 & value9 \\\\\r\n"
            "\\hline\r\n"
            "\\end{tabular}"
        )

        hrule_options: dict[str, HRuleStyle] = {"hrules": HRuleStyle.ALL}
        assert helper_table.get_latex_string(format=True, **hrule_options) == (
            "\\begin{tabular}{|c|c|c|c|}\r\n"
            "\\hline\r\n"
            " & Field 1 & Field 2 & Field 3 \\\\\r\n"
            "\\hline\r\n"
            "1 & value 1 & value2 & value3 \\\\\r\n"
            "\\hline\r\n"
            "4 & value 4 & value5 & value6 \\\\\r\n"
            "\\hline\r\n"
            "7 & value 7 & value8 & value9 \\\\\r\n"
            "\\hline\r\n"
            "\\end{tabular}"
        )

    def test_latex_output_header(self, helper_table: PrettyTable) -> None:
        assert helper_table.get_latex_string(format=True, hrules=HRuleStyle.HEADER) == (
            "\\begin{tabular}{|c|c|c|c|}\r\n"
            " & Field 1 & Field 2 & Field 3 \\\\\r\n"
            "\\hline\r\n"
            "1 & value 1 & value2 & value3 \\\\\r\n"
            "4 & value 4 & value5 & value6 \\\\\r\n"
            "7 & value 7 & value8 & value9 \\\\\r\n"
            "\\end{tabular}"
        )

    def test_internal_border_preserved_latex(self, helper_table: PrettyTable) -> None:
        helper_table.border = False
        helper_table.format = True
        helper_table.preserve_internal_border = True

        assert helper_table.get_latex_string().strip() == (
            "\\begin{tabular}{c|c|c|c}\r\n"
            " & Field 1 & Field 2 & Field 3 \\\\\r\n"
            "1 & value 1 & value2 & value3 \\\\\r\n"
            "4 & value 4 & value5 & value6 \\\\\r\n"
            "7 & value 7 & value8 & value9 \\\\\r\n"
            "\\end{tabular}"
        )



from __future__ import annotations

from prettytable import PrettyTable, from_json


class TestJSONOutput:
    def test_json_output(self, helper_table: PrettyTable) -> None:
        result = helper_table.get_json_string()
        assert (
            result.strip()
            == """
[
    [
        "",
        "Field 1",
        "Field 2",
        "Field 3"
    ],
    {
        "": 1,
        "Field 1": "value 1",
        "Field 2": "value2",
        "Field 3": "value3"
    },
    {
        "": 4,
        "Field 1": "value 4",
        "Field 2": "value5",
        "Field 3": "value6"
    },
    {
        "": 7,
        "Field 1": "value 7",
        "Field 2": "value8",
        "Field 3": "value9"
    }
]""".strip()
        )
        options = {"fields": ["Field 1", "Field 3"]}
        result = helper_table.get_json_string(**options)
        assert (
            result.strip()
            == """
[
    [
        "Field 1",
        "Field 3"
    ],
    {
        "Field 1": "value 1",
        "Field 3": "value3"
    },
    {
        "Field 1": "value 4",
        "Field 3": "value6"
    },
    {
        "Field 1": "value 7",
        "Field 3": "value9"
    }
]""".strip()
        )

    def test_json_output_options(self, helper_table: PrettyTable) -> None:
        result = helper_table.get_json_string(
            header=False, indent=None, separators=(",", ":")
        )
        assert (
            result
            == """[{"":1,"Field 1":"value 1","Field 2":"value2","Field 3":"value3"},"""
            """{"":4,"Field 1":"value 4","Field 2":"value5","Field 3":"value6"},"""
            """{"":7,"Field 1":"value 7","Field 2":"value8","Field 3":"value9"}]"""
        )


class TestJSONConstructor:
    def test_json_and_back(self, city_data: PrettyTable) -> None:
        json_string = city_data.get_json_string()
        new_table = from_json(json_string)
        assert new_table.get_string() == city_data.get_string()



from __future__ import annotations

from prettytable import PrettyTable, TableStyle


class TestRowEndSection:
    EXPECTED_RESULT = """
┌──────────┬──────────┬──────────┐
│ Field 1  │ Field 2  │ Field 3  │
├──────────┼──────────┼──────────┤
│ value 4  │ value 5  │ value 6  │
│ value 7  │ value 8  │ value 9  │
├──────────┼──────────┼──────────┤
│ value 10 │ value 11 │ value 12 │
└──────────┴──────────┴──────────┘
""".strip()

    TEST_ROWS = [
        ["value 4", "value 5", "value 6"],
        ["value 7", "value 8", "value 9"],
        ["value 10", "value 11", "value 12"],
    ]

    def test_row_end_section_via_argument(self) -> None:
        table = PrettyTable()
        table.set_style(TableStyle.SINGLE_BORDER)
        table.add_row(self.TEST_ROWS[0])
        table.add_row(self.TEST_ROWS[1], divider=True)
        table.add_row(self.TEST_ROWS[2])
        assert table.get_string().strip() == self.EXPECTED_RESULT

    def test_row_end_section_via_method(self) -> None:
        table = PrettyTable()
        table.set_style(TableStyle.SINGLE_BORDER)
        table.add_row(self.TEST_ROWS[0])
        table.add_row(self.TEST_ROWS[1])
        table.add_divider()
        table.add_row(self.TEST_ROWS[2])
        assert table.get_string().strip() == self.EXPECTED_RESULT

    def test_add_rows_divider(self) -> None:
        """A table created with two add_rows calls, one with divider=True has a
        divider"""
        table = PrettyTable()
        table.set_style(TableStyle.SINGLE_BORDER)
        table.add_rows(self.TEST_ROWS[0:2], divider=True)
        table.add_rows(self.TEST_ROWS[2:])
        assert table.get_string().strip() == self.EXPECTED_RESULT


class TestClearing:
    def test_clear_rows(self, helper_table: PrettyTable) -> None:
        helper_table.add_row([0, "a", "b", "c"], divider=True)
        helper_table.clear_rows()
        assert helper_table.rows == []
        assert helper_table.dividers == []
        assert helper_table.field_names == ["", "Field 1", "Field 2", "Field 3"]

    def test_clear(self, helper_table: PrettyTable) -> None:
        helper_table.add_row([0, "a", "b", "c"], divider=True)
        helper_table.clear()
        assert helper_table.rows == []
        assert helper_table.dividers == []
        assert helper_table.field_names == []
